#!/usr/bin/env python

import string
import os
import sys

def msg_checking(msg):
    print "Checking", msg, "...",

def execute(cmd, display = 0):
    if display:
        print cmd
    return os.system(cmd)

def run_test(input, flags = ''):
    try:
        f = open('_temp.c', 'w')
        f.write(input)
        f.close()
        compile_cmd = '%s -o _temp _temp.c %s' % (os.environ.get('CC', 'cc'),
                                                  flags)
        if not execute(compile_cmd):
            execute('./_temp')
    finally:
        execute('rm -f _temp.c _temp')
    
anx_test_program = '''
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <annodex/annodex.h>

int main ()
{
  system("touch conf.anxtest");
  return 0;
}
'''

def find_annodex(anx_prefix = '/usr/local', enable_anxtest = 1):
    """A rough translation of ogg.m4"""

    anx_cflags = []
    anx_libs = []
    
    anx_include_dir = anx_prefix + '/include'
    anx_lib_dir = anx_prefix + '/lib'
    anx_libs = 'annodex'

    msg_checking('for Annodex')

    if enable_anxtest:
        execute('rm -f conf.anxtest', 0)

        try:
            run_test(anx_test_program, flags="-I" + anx_include_dir)
            if not os.path.isfile('conf.anxtest'):
                raise RuntimeError, "Did not produce output"
            execute('rm conf.anxtest', 0)
            
        except:
            print "test program failed"
            return None

    print "success"

    return {'annodex_libs' : anx_libs,
            'annodex_lib_dir' : anx_lib_dir,
            'annodex_include_dir' : anx_include_dir}

def write_data(data):
    f = open('Setup', 'w')
    for item in data.items():
        f.write('%s = %s\n' % item)
    f.close()
    print "Wrote Setup file"
            
def print_help():
    print '''%s
    --prefix      Give the prefix in which annodex was installed.''' % sys.argv[0]
    sys.exit(0)

def parse_args():
    data = {}
    argv = sys.argv
    for pos in range(len(argv)):
        if argv[pos] == '--help':
            print_help()
        if argv[pos] == '--prefix':
            pos = pos + 1
            if len(argv) == pos:
                print "Prefix needs an argument"
                sys.exit(1)
            data['prefix'] = argv[pos]

    return data
    
def main():
    args = parse_args()
    prefix = args.get('prefix', '/usr/local')
    print "Prefix: ", prefix
    data = find_annodex(anx_prefix = prefix)
    if not data:
        print "Config failure"
        sys.exit(1)
    write_data(data)

if __name__ == '__main__':
    main()
